(* Based on: http://www.codeproject.com/KB/recipes/Levenshtein.aspx?fid=282908&fr=11&df=90&mpp=25&noise=3&sort=Position&view=Quick *)
(* Step	Description *)
(* 1	Set n to be the length of s. ("GUMBO") *)
(* Set m to be the length of t. ("GAMBOL") *)
(* If n = 0, return m and exit. *)
(* If m = 0, return n and exit. *)
(* Construct two vectors, v0[m+1] and v1[m+1], containing 0..m elements. *)
(* 2	Initialize v0 to 0..m. *)
(* 3	Examine each character of s (i from 1 to n). *)
(* 4	Examine each character of t (j from 1 to m). *)
(* 5	If s[i] equals t[j], the cost is 0. *)
(* If s[i] is not equal to t[j], the cost is 1. *)
(* 6	Set cell v1[j] equal to the minimum of: *)
(* a. The cell immediately above plus 1: v1[j-1] + 1. *)
(* b. The cell immediately to the left plus 1: v0[j] + 1. *)
(* c. The cell diagonally above and to the left plus the cost: v0[j-1] + cost. *)
(* 7	After the iteration steps (3, 4, 5, 6) are complete, the distance is found in the cell v1[m]. *)

let min2 (a : int) b = if a < b then a else b

let min3 (a : int) b c = min2 a (min2 b c)

let str_of_int_array a =
  "[|" ^ String.concat ";" (List.map string_of_int (Array.to_list a)) ^ "|]"

let leven s t =
  let n = String.length s in
  let m = String.length t in
    if n = 0 then m
    else if m = 0 then n
    else
      let v0 = Array.init (m + 1) (fun i -> i) in
      let v1 = Array.make (m + 1) 0 in
	for i = 1 to n do
	  v1.(0) <- i;
	  for j = 1 to m do
	    let cost =
	      if s.[i - 1] = t.[j - 1] then 0
	      else 1
	    in
	      v1.(j) <- 
		(min3
		   (v1.(j - 1) + 1)
		   (v0.(j) + 1)
		   (v0.(j - 1) + cost)
		);
	  done;
	  for c = 0 to m do
	    v0.(c) <- v1.(c)
	  done
	done;
	v1.(m)
