(* Copyright 2009 Erkki Seppälä <flux@modeemi.fi> *)
(* Refer to LICENSE.ganame for licensing details *)

open ExtList
open ExtString
open Genome

let main () =
  let examples = ref [] in
  let inputs = ref [] in
  let no_act = ref false in
  let fast_stop = ref false in
  let args = Arg.align ["--", Arg.Rest (fun arg -> inputs := arg::!inputs), " List input files after this";
			"-n", Arg.Set no_act, " Don't act, just simulate the behavior";
			"--noact", Arg.Set no_act, " Don't act, just simulate the behavior";
			"-f", Arg.Set fast_stop, " Fast stop; if a decent solution is found, stop the evolution faster";
		       ] in
  let usage = "Ganame renames files based on examples. Copyright Erkki Seppl <flux@modeemi.fi> 2009.\nganame [example1] [example2] -- *.ext\n" in
  let () = Arg.parse args (fun arg -> examples := arg::!examples) usage in
  let fast_stop = !fast_stop in
  let examples = List.rev !examples in
  let examples = List.map (fun s -> String.split s "=") examples in
    match examples with 
      | [] -> Arg.usage args usage
      | _ -> 
	  let inputs = List.rev !inputs in
	    match generate_mapping ~fast_stop examples inputs with
	      | None -> 
		  Printf.printf "Failed to produce a mapping\n%!";
		  exit 1
	      | Some [] -> 
		  exit 0
	      | Some _ when !no_act ->
		  ()
	      | Some mapping ->
		  Printf.printf "Mapping produced succesfully. Rename [y/n]? %!";
		  match input_line stdin with
		    | "y" | "yes" | "Y" | "YES" -> 
			List.iter (fun (a, b) -> Unix.rename a b) mapping;
			exit 0
		    | _ ->
			Printf.printf "Not renaming files\n";
			exit 1

let () = 
  if not !Sys.interactive then
    main ()

