(* Copyright 2009 Erkki Seppälä <flux@modeemi.fi> *)
(* Refer to LICENSE.ganame for licensing details *)

open ExtString

let quote = Netencoding.Html.encode_from_latin1

let operations = ref 0
let operations_lock = Mutex.create ()

let incr_operations_if_zero () =
  Mutex.lock operations_lock;
  let success = 
    if !operations = 0 then
      ( incr operations;
	true )
    else
      false
  in
    Mutex.unlock operations_lock;
    success

let o (cgi : Netcgi.cgi) fmt = Printf.ksprintf cgi#out_channel#output_string fmt

let decr_operations () =
  Mutex.lock operations_lock;
  decr operations;
  Mutex.unlock operations_lock

let perform_operations (cgi : Netcgi.cgi) examples inputs =
  o cgi "<html><pre>";
  let mapping = Genome.generate_mapping ~output:cgi#out_channel#output_string examples inputs in
    o cgi "</pre>";
    ( match mapping with
	| None -> o cgi "<h2>Cannot produce mapping</h2>"
	| Some mapping ->
	    o cgi "Mapping:";
	    o cgi "<table>";
	    List.iter (fun (s1, s2) -> o cgi "<tr><td>%s</td><td>%s</td></tr>" (quote s1) (quote s2)) mapping;
	    o cgi "</table>";

    );
    o cgi "</html>"

let query_page (cgi : Netcgi.cgi) =
  o cgi "\
<html>
<h1>Ganame Demo Page</h1>
<form method=\"GET\">
<p>Enter examples in form aaa=AAA, one per line.<br>
<textarea name=\"examples\" type=\"textbox\" cols=\"40\" rows=\"10\"></textarea>
<p>Enter inputs, one per line.<br>
<textarea name=\"input\" type=\"textbox\" cols=\"40\" rows=\"10\"></textarea>
<br><input name=\"submit\" value=\"Submit\" type=\"submit\">
</form>
</html>"

let busy_page (cgi : Netcgi.cgi) =
  cgi#set_header
    ~cache:`No_cache
    ~status:`Service_unavailable
    ~content_type:"text/html; charset=iso-8859-15" 
    ();
  cgi#out_channel#output_string "<html><h1>Sorry, I'm busy. Only one request concurrently.</h1></html>"

let page (cgi : Netcgi.cgi) =
  let default_header () = 
    cgi#set_header
      ~cache:`No_cache
      ~content_type:"text/html; charset=iso-8859-15" 
      ()
  in
    if cgi#argument_exists "input" && cgi#argument_exists "examples" then
      if incr_operations_if_zero () then
	Std.finally decr_operations 
	  (fun () ->
	     try
	       default_header ();
	       let examples = List.map (fun s -> String.split s "=") (Pcre.split ~pat:"\r\n" (cgi#argument "examples")#value) in
	       let inputs = Pcre.split ~pat:"\r\n" (cgi#argument "input")#value in
		 perform_operations cgi examples inputs;
	     with exn ->
	       o cgi "<html>Error: %s</html>" (Printexc.to_string exn);
	  ) ()
      else
	busy_page cgi
    else
      ( default_header ();
	query_page cgi );
    cgi#out_channel#commit_work ()


let main () = Netcgi_scgi_threaded.run ~port:4243 page

let _ = main ()
