open OUnit

let string_of_mapping m = String.concat " " (List.map (fun (a, b) -> a ^ "=" ^ b) m)

let mk_store () =
  let q = ref [] in
  (object
     method log msg = q := msg::!q
     method get = String.concat "\n" (List.rev (""::!q))
   end)

let test label examples inputs =
  label >:: 
    fun () ->
      let store = mk_store () in
	match Genome.generate_mapping ~output:store#log examples (List.map fst (Option.default [] inputs)), inputs with
	  | None, None -> ()
	  | Some x, Some y when x = y -> ()
	  | Some x, Some y (* when x <> y *) -> Printf.ksprintf failwith "Invalid mapping produced: %s vs expected %s. Log:\n%s" (string_of_mapping x) (string_of_mapping y) store#get
	  | Some x, None -> Printf.ksprintf failwith "Mapping produced when none expected: %s. Log:\n%s" (string_of_mapping x) store#get
	  | None, Some x -> Printf.ksprintf failwith "No mapping produced when one expected: expected %s. Log:\n%s" (string_of_mapping x) store#get

let tests =
  TestList
    [test "Uppercase first" ["aaa", "Aaa"; "bbbb", "Bbbb"] (Some ["ccc", "Ccc"; "dddd", "Dddd"]);
     test "Lowercase all" ["AAA", "aaa"; "BBBB", "bbbb"] (Some ["CCC", "ccc"; "DDDD", "dddd"]);
     test "Img example" ["IMG_1234.JPG", "img_1234.JPG"] (Some ["IMG_2345.JPG", "img_2345.JPG"]);
     test "Insert" ["IMG_0001.JPG", "Image-0001.jpg"; "IMG_2345.JPG", "Image-2345.jpg"] (Some ["IMG_3456.JPG", "Image-3456.jpg"]);
     test "Mapping" ["Kissa", "Ki55a"; "Hevonen", "Hev0nen"; "Mesoa", "Me50a"] (Some ["Leet salakoodi", "Leet 5alak00di"]);
    ]

let _ = ignore (run_test_tt ~verbose:true tests) 
