open ExtList

let string_of_array_rev ar =
  let len = Array.length ar in
  let s = String.create len in
    for c = 0 to len - 1 do
      s.[Array.length ar - 1 - c] <- ar.(c)
    done;
    s

let common_matrix ?(square=false) s1 s2 = 
  let m = 
    if square then 
      let size = max (String.length s1) (String.length s2) in
	Array.make_matrix size size 0
    else
      Array.make_matrix (String.length s1) (String.length s2) 0 
  in
    for s1'c = 0 to String.length s1 - 1 do
      for s2'c = 0 to String.length s2 - 1 do
	if s1.[s1'c] = s2.[s2'c] then
	  m.(s1'c).(s2'c) <- 
	    1 + (if s1'c > 0 && s2'c > 0 then
		   m.(s1'c - 1).(s2'c - 1)
		 else
		   0
		)
      done
    done;
    m

let common_words s1 s2 =
  let m = common_matrix s1 s2 in
    let results = ref [] in
    for s1'c = String.length s1 - 1 downto 0 do
      for s2'c = String.length s2 - 1 downto 0 do
	let len = m.(s1'c).(s2'c) in
	  for c = 0 to len - 1 do
	    m.(s1'c - c).(s2'c - c) <- 0
	  done;
	  if len >= 2 then
	    results :=
	      string_of_array_rev (Array.init len (fun c -> s1.[s1'c - c]))
	    ::!results
      done
    done;
      !results

let dump_matrix m =
  for y = 0 to Array.length m - 1 do
    for x = 0 to Array.length m.(y) - 1 do
      print_string (string_of_int m.(y).(x)) 
    done;
    print_newline ()
  done

let find_consec cond seq =
  let rec scanning at seq results =
    match seq with
      | [] -> List.rev results
      | x::rest when cond x -> found_one (at + 1) rest results at
      | _::rest -> scanning (at + 1) rest results
  and found_one at seq results begun =
    match seq with
      | [] -> scanning at seq ((begun, at - begun)::results)
      | x::rest when cond x -> found_one (at + 1) rest results begun
      | _::rest -> scanning (at + 1) rest ((begun, at - begun)::results)
  in
    scanning 0 seq []

let new_words s1 s2 =
  let m = common_matrix ~square:true s1 s2 in
  let size = Array.length m in
  let results = ref [] in
  let visit_border s1'c0 s2'c0 =
    Printf.printf "Visit %d,%d\n%!" s1'c0 s2'c0;
    let rec loop s1'c s2'c s2'prev accu len =
      if s1'c >= 0 && s2'c >= 0 then
	loop (s1'c - 1) (s2'c - 1) s2'c (m.(s1'c).(s2'c)::accu) (len + 1)
      else
	(accu, s2'prev, len)
    in
    let (sequence, offset, max_len) = loop s1'c0 s2'c0 s2'c0 [] 0 in
    let () = Printf.printf "Sequence = %s\n" (String.concat "," (List.map string_of_int sequence)) in
    let () = Printf.printf "s2'c0 = %d, offset = %d\n" s2'c0 offset in
    let consecutive = 
      List.map (fun (ofs, len) -> (ofs + offset, len)) 
	(find_consec ((=) 0) sequence)
    in
      if List.exists
	(fun (ofs, len) ->
	   Printf.printf "ofs (%d), ofs + len - offset (%d) max_len (%d)\n"
	     ofs (ofs + len - offset) max_len;
	   ofs <> offset || ofs + len - offset <> max_len
	)
	consecutive then
	  let consecutive_strings =
	    List.map 
	      (fun (ofs, len) -> 
		 Printf.printf "s2=%s ofs=%d len=%d\n" s2 ofs len;
		 String.sub s2 ofs len
	      )
	      consecutive
	  in
	    results:=List.rev_append !results consecutive_strings
  in
    for s2'c = 0 to String.length s2 - 1 do
      visit_border s2'c (String.length s2 - 1)
    done;
    for s2'c = 0 to String.length s2 - 2 do
      visit_border (String.length s2 - 1) s2'c 
    done;
    !results

let common_words_list = function
  | [] -> []
  | x::[] -> [x]
  | word::word'::rest ->
      List.unique (
	List.fold_left
	  (fun words word ->
	     List.concat (List.map (common_words word) words)
	  )
	  (common_words word word')
	  rest
      )
